/* +---------------------------------------------------------------------------+
   |          The Mobile Robot Programming Toolkit (MRPT) C++ library          |
   |                                                                           |
   |                   http://mrpt.sourceforge.net/                            |
   |                                                                           |
   |   Copyright (C) 2005-2009  University of Malaga                           |
   |                                                                           |
   |    This software was written by the Machine Perception and Intelligent    |
   |      Robotics Lab, University of Malaga (Spain).                          |
   |    Contact: Jose-Luis Blanco  <jlblanco@ctima.uma.es>                     |
   |                                                                           |
   |  This file is part of the MRPT project.                                   |
   |                                                                           |
   |     MRPT is free software: you can redistribute it and/or modify          |
   |     it under the terms of the GNU General Public License as published by  |
   |     the Free Software Foundation, either version 3 of the License, or     |
   |     (at your option) any later version.                                   |
   |                                                                           |
   |   MRPT is distributed in the hope that it will be useful,                 |
   |     but WITHOUT ANY WARRANTY; without even the implied warranty of        |
   |     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         |
   |     GNU General Public License for more details.                          |
   |                                                                           |
   |     You should have received a copy of the GNU General Public License     |
   |     along with MRPT.  If not, see <http://www.gnu.org/licenses/>.         |
   |                                                                           |
   +---------------------------------------------------------------------------+ */

#include <mrpt/precomp_core.h>  // Only for precomp. headers, include all libmrpt-core headers.


#include <mrpt/opengl/CPointCloudColoured.h>
#include <mrpt/slam/CPointsMap.h>
#include <mrpt/slam/CColouredPointsMap.h>
#include <mrpt/math/utils.h>

#include "opengl_internals.h"

using namespace mrpt;
using namespace mrpt::opengl;
using namespace mrpt::utils;
using namespace mrpt::slam;
using namespace mrpt::math;
using namespace std;

IMPLEMENTS_SERIALIZABLE( CPointCloudColoured, CRenderizable, mrpt::opengl )

/*---------------------------------------------------------------
							render
  ---------------------------------------------------------------*/
void   CPointCloudColoured::render() const
{
#if MRPT_HAS_OPENGL_GLUT

	if ( m_color_A != 1.0 )
	{
		//glDisable(GL_DEPTH_TEST);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}

    glPointSize( m_pointSize );

    glBegin( GL_POINTS );

    glColor4f( m_color_R,m_color_G,m_color_B,m_color_A );

    for (const_iterator i=begin();i!=end();i++)
    {
		glColor4f( i->R,i->G,i->B,m_color_A );
		glVertex3f( i->x,i->y,i->z );
    }

    glEnd();

	if ( m_color_A != 1.0 )
	{
		//glEnable(GL_DEPTH_TEST);
		glDisable(GL_BLEND);
	}

	checkOpenGLError();
#endif
}

/*---------------------------------------------------------------
   Implements the writing to a CStream capability of
     CSerializable objects
  ---------------------------------------------------------------*/
void  CPointCloudColoured::writeToStream(CStream &out,int *version) const
{

	if (version)
		*version = 0;
	else
	{
		writeToStreamRender(out);
		m_points.write(out);
		out << m_pointSize;
	}
}

/*---------------------------------------------------------------
	Implements the reading from a CStream capability of
		CSerializable objects
  ---------------------------------------------------------------*/
void  CPointCloudColoured::readFromStream(CStream &in,int version)
{
	switch(version)
	{
	case 0:
		{
			readFromStreamRender(in);
			m_points.read(in);
			in >> m_pointSize;
		} break;
	default:
		MRPT_THROW_UNKNOWN_SERIALIZATION_VERSION(version)
	};
}

/*---------------------------------------------------------------
						loadFromPointsMap
---------------------------------------------------------------*/
void  CPointCloudColoured::loadFromPointsMap( const slam::CPointsMap *themap)
{
	if (IS_CLASS(themap,CColouredPointsMap))
	{
		// Copy points & colors:
		const CColouredPointsMap* m = static_cast<const CColouredPointsMap*>(themap);

		size_t N = m->size();
		m_points.resize(N);
		for (size_t i=0;i<N;i++)
		{
			m->getPoint(i,
				m_points[i].x,
				m_points[i].y,
				m_points[i].z,
				m_points[i].R,
				m_points[i].G,
				m_points[i].B );
		}
	}
	else
	{
		// Default colors:
		vector_float	xs,ys,zs;
		themap->getAllPoints(xs,ys,zs);

		size_t N = xs.size();
		m_points.resize(N);
		for (size_t i=0;i<N;i++)
		{
			m_points[i].x = xs[i];
			m_points[i].y = ys[i];
			m_points[i].z = zs[i];
			m_points[i].R = m_color_R;
			m_points[i].G = m_color_G;
			m_points[i].B = m_color_B;
		}
	}
}

CStream& mrpt::opengl::operator >> (CStream& in,  CPointCloudColoured::TPointColour &o)
{
	in >> o.x >> o.y >> o.z >> o.R >> o.G >> o.B;
	return in;
}


CStream& mrpt::opengl::operator << (CStream& out, const CPointCloudColoured::TPointColour &o)
{
	out << o.x << o.y << o.z << o.R << o.G << o.B;
	return out;
}
