/* +---------------------------------------------------------------------------+
   |          The Mobile Robot Programming Toolkit (MRPT) C++ library          |
   |                                                                           |
   |                   http://mrpt.sourceforge.net/                            |
   |                                                                           |
   |   Copyright (C) 2005-2009  University of Malaga                           |
   |                                                                           |
   |    This software was written by the Machine Perception and Intelligent    |
   |      Robotics Lab, University of Malaga (Spain).                          |
   |    Contact: Jose-Luis Blanco  <jlblanco@ctima.uma.es>                     |
   |                                                                           |
   |  This file is part of the MRPT project.                                   |
   |                                                                           |
   |     MRPT is free software: you can redistribute it and/or modify          |
   |     it under the terms of the GNU General Public License as published by  |
   |     the Free Software Foundation, either version 3 of the License, or     |
   |     (at your option) any later version.                                   |
   |                                                                           |
   |   MRPT is distributed in the hope that it will be useful,                 |
   |     but WITHOUT ANY WARRANTY; without even the implied warranty of        |
   |     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         |
   |     GNU General Public License for more details.                          |
   |                                                                           |
   |     You should have received a copy of the GNU General Public License     |
   |     along with MRPT.  If not, see <http://www.gnu.org/licenses/>.         |
   |                                                                           |
   +---------------------------------------------------------------------------+ */

#include <mrpt/precomp_core.h>  // Only for precomp. headers, include all libmrpt-core headers. 



#include <mrpt/poses/CPoseOrPoint.h>
#include <mrpt/math/utils.h>

using namespace mrpt;
using namespace mrpt::poses;
using namespace mrpt::math;

IMPLEMENTS_VIRTUAL_SERIALIZABLE(CPoseOrPoint, CSerializable,mrpt::poses)

/*---------------------------------------------------------------
							norm
	Returns the euclidean norm of vector:
  ---------------------------------------------------------------*/
double  CPoseOrPoint::norm() const
{
	if (is3D)
			return sqrt(x*x+y*y+z*z);
	else	return sqrt(x*x+y*y);
}

/*---------------------------------------------------------------
					getInverseHomogeneousMatrix
  ---------------------------------------------------------------*/
void CPoseOrPoint::getInverseHomogeneousMatrix(CMatrixDouble &out_HM)const
{
	const math::CMatrixDouble * ptrHM;
	getHomogeneousMatrix(ptrHM);
	math::homogeneousMatrixInverse( *ptrHM, out_HM );
}
