/* +---------------------------------------------------------------------------+
   |          The Mobile Robot Programming Toolkit (MRPT) C++ library          |
   |                                                                           |
   |                   http://mrpt.sourceforge.net/                            |
   |                                                                           |
   |   Copyright (C) 2005-2009  University of Malaga                           |
   |                                                                           |
   |    This software was written by the Machine Perception and Intelligent    |
   |      Robotics Lab, University of Malaga (Spain).                          |
   |    Contact: Jose-Luis Blanco  <jlblanco@ctima.uma.es>                     |
   |                                                                           |
   |  This file is part of the MRPT project.                                   |
   |                                                                           |
   |     MRPT is free software: you can redistribute it and/or modify          |
   |     it under the terms of the GNU General Public License as published by  |
   |     the Free Software Foundation, either version 3 of the License, or     |
   |     (at your option) any later version.                                   |
   |                                                                           |
   |   MRPT is distributed in the hope that it will be useful,                 |
   |     but WITHOUT ANY WARRANTY; without even the implied warranty of        |
   |     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         |
   |     GNU General Public License for more details.                          |
   |                                                                           |
   |     You should have received a copy of the GNU General Public License     |
   |     along with MRPT.  If not, see <http://www.gnu.org/licenses/>.         |
   |                                                                           |
   +---------------------------------------------------------------------------+ */

#include <mrpt/precomp_core.h>  // Only for precomp. headers, include all libmrpt-core headers. 


#include <mrpt/utils/CMessageQueue.h>

using namespace mrpt::utils;
using namespace mrpt::synch;

/*---------------------------------------------------------------
				Constructor
 ---------------------------------------------------------------*/
CMessageQueue::CMessageQueue()
{

}

/*---------------------------------------------------------------
				Destructor
 ---------------------------------------------------------------*/
CMessageQueue::~CMessageQueue()
{
	clear();
}

/*---------------------------------------------------------------
				clear
 ---------------------------------------------------------------*/
void CMessageQueue::clear()
{
	CCriticalSectionLocker locker( &m_csQueue );

	while (!m_msgs.empty())
	{
		delete m_msgs.front();
		m_msgs.pop();
	}
}

/*---------------------------------------------------------------
				push
 ---------------------------------------------------------------*/
void CMessageQueue::push( CMRPTMessage *msg )
{
	CCriticalSectionLocker locker( &m_csQueue );

	m_msgs.push( msg );
}

/*---------------------------------------------------------------
				get
 ---------------------------------------------------------------*/
CMRPTMessage *CMessageQueue::get( )
{
	CCriticalSectionLocker locker( &m_csQueue );

	if (m_msgs.empty())
		return NULL;
	else
	{
		CMRPTMessage *ret = m_msgs.front();
		m_msgs.pop();
		return ret;
	}
}

/*---------------------------------------------------------------
				empty
 ---------------------------------------------------------------*/
bool CMessageQueue::empty() const
{
	CCriticalSectionLocker locker( &m_csQueue );
	return m_msgs.empty();
}
